package de.fhhannover.inform.trust.ifmapj_examples;

import java.util.Collection;

import de.fhhannover.inform.trust.ifmapj.channel.ARC;
import de.fhhannover.inform.trust.ifmapj.channel.SSRC;
import de.fhhannover.inform.trust.ifmapj.exception.EndSessionException;
import de.fhhannover.inform.trust.ifmapj.exception.IfmapErrorResult;
import de.fhhannover.inform.trust.ifmapj.exception.IfmapException;
import de.fhhannover.inform.trust.ifmapj.exception.InitializationException;
import de.fhhannover.inform.trust.ifmapj.messages.ResultItem;
import de.fhhannover.inform.trust.ifmapj.messages.SearchResult;

/**
 * Hello world!
 *
 */
public class IfmapJExamples 
{
	// some examples want to do some sleeping
	private static final int SLEEP_TIME = 50; // ms
	
    public static void main( String[] args ) {
        System.out.println( "=== IFMAPJ EXAMPLES ===" );
        try {
        	new IdentityTesting().run(args);
        	/*
        	new PollExample().run(args);
        	new SwitchSSRC().run(args);
        	new SubscriptionSearchResults().run(args);
        	new PollWithSupgraphAddingRemovingExample().run(args);
        	new PollForEvents().run(args);
        	new SessionExample().run(args);
	        new PublishExample().run(args);
	        new LoopLink().run(args);
	        new PublishExample2().run(args);
	        */
	} catch (InitializationException e) {
		System.err.println(e.getDescription());
		System.err.println(e.getMessage());
	}
    }
    
    static void sleepSomeTime() {
    	try {
    		Thread.sleep(SLEEP_TIME);
    	} catch (InterruptedException e) {
    		// ignore
    	}
    };
    
    /**
     * The last poll on an {@link ARC} channel might lead to an ErrorResult or
     * EndSessionResult at or while the poll call is done.
     * Unify this as the polling examples are using this.
     * 
     * @param arc
     * @throws IfmapException 
     */
    static void doLastPoll(ARC arc) throws IfmapException {
    	String finalMsg = null;
		try {
			arc.poll();
		} catch (EndSessionException e) {
			finalMsg = "EndSessionResult";
		} catch (IfmapErrorResult e) {
			finalMsg = e.toString();
		} finally {
			System.out.println("[poll-thread] Last poll ... Resulted in " + finalMsg);
			arc.closeTcpConnection();
		}
    }
    
    static void doLastEndSession(SSRC ssrc) throws IfmapException {
    	doLastEndSession(null, ssrc);
    }

	public static void doLastEndSession(String prefix, SSRC ssrc) throws IfmapException {
    	String finalMsg = "OK";
    	prefix = (prefix != null) ? prefix + " " : "";
    	try {
			ssrc.endSession();
    	} catch (IfmapErrorResult err) {
    		finalMsg = err.toString();
    	} finally {
    		System.out.println(prefix + "endSession ... " + finalMsg);
			ssrc.closeTcpConnection();
    	}
	}
	

	/**
	 * Helper to print out ResultItems...
	 */
	static void printSearchResult(SearchResult sres) {
		printSearchResult("searchResult", sres);
	}

	public static void printSearchResult(String resultName, SearchResult sres) {
		System.out.println("-------- print " + resultName + " name=" + sres.getName());
		Collection<ResultItem> ris = sres.getResultItems();
		int i = 0;
		for (ResultItem ri : ris) {
			System.out.println("Item " + i + ") " + ri);
			i++;
		}
		System.out.println("-------------------------------------------------");
	}
}
